﻿using System;
using System.Windows;

//------------------------------------
// This extension adds WebSocket support to C#/XAML for HTML5 (www.cshtml5.com)
//
// It requires Beta 7.2 or newer.
//
// This project is licensed under The open-source MIT license:
// https://opensource.org/licenses/MIT
//
// Copyright 2016 Userware / CSHTML5
//------------------------------------

namespace CSHTML5.Extensions.WebSockets
{
    public class WebSocket
    {
        object _referenceToTheJavaScriptWebSocketObject;

        public event EventHandler OnOpen;
        public event EventHandler OnClose;
        public event EventHandler<OnMessageEventArgs> OnMessage;
        public event EventHandler<OnErrorEventArgs> OnError;

        public WebSocket(string uri)
        {
            // var exampleSocket = new WebSocket("ws://www.example.com/socketserver", "protocolOne");
            // On return, exampleSocket.readyState is CONNECTING. 
            // The readyState will become OPEN once the connection is ready to transfer data.

            _referenceToTheJavaScriptWebSocketObject = CSHTML5.Interop.ExecuteJavaScript("new WebSocket($0)", uri);

            // var _refWebSockObj =
            CSHTML5.Interop.ExecuteJavaScript(
                    @"$0.onopen = $1; $0.onclose = $2; $0.onmessage = $3; $0.onerror = $4",
                _referenceToTheJavaScriptWebSocketObject,
                (Action<object>)this.OnOpenCallback,
                (Action<object>)this.OnCloseCallback,
                (Action<object>)this.OnMessageCallback,
                (Action<object>)this.OnErrorCallback);
        }

        public int Get_readyState()
        {
            var _readyState = CSHTML5.Interop.ExecuteJavaScript(@"$0.readyState", _referenceToTheJavaScriptWebSocketObject);

            /*  CONNECTING	0	The connection is not yet open.
                OPEN	1	The connection is open and ready to communicate.
                CLOSING	2	The connection is in the process of closing.
                CLOSED	3	The connection is closed or couldn't be opened.
            */
            return Convert.ToInt32(_readyState);
        }

        public void Send(string message)
        {
            CSHTML5.Interop.ExecuteJavaScript("$0.send($1)", _referenceToTheJavaScriptWebSocketObject, message);
        }

        public void Close()
        {
            CSHTML5.Interop.ExecuteJavaScript("$0.close()", _referenceToTheJavaScriptWebSocketObject);
        }

        void OnOpenCallback(object e)
        {
            if (this.OnOpen != null)
                OnOpen(this, new EventArgs());
        }

        void OnCloseCallback(object e)
        {
            if (this.OnClose != null)
                OnClose(this, new EventArgs());
        }

        void OnMessageCallback(object e)
        {
            var data = Convert.ToString(CSHTML5.Interop.ExecuteJavaScript("$0.data", e));

            if (this.OnMessage != null)
                OnMessage(this, new OnMessageEventArgs(data));
        }

        void OnErrorCallback(object e)
        {
            // var data = Convert.ToString(CSHTML5.Interop.ExecuteJavaScript("$0.data", e));
            var data = "OnErrorCallback";

            if (this.OnError != null)
                OnError(this, new OnErrorEventArgs(data));
        }
    }

    public class OnMessageEventArgs : EventArgs
    {
        public readonly string Data;

        public OnMessageEventArgs(string data)
        {
            this.Data = data;
        }
    }

    public class OnErrorEventArgs : EventArgs
    {
        public readonly string Data;

        public OnErrorEventArgs(string data)
        {
            this.Data = data;
        }
    }
}